# src/file_conversor/backend/gui/_components/_class.py

def class_is_pulled_left():
    return "is-pulled-left"


def class_is_pulled_right():
    return "is-pulled-right"


def class_is_clickable():
    return "is-clickable"


def class_is_unselectable():
    return "is-unselectable"


def class_is_relative():
    return "is-relative"


__all__ = [
    'class_is_pulled_left',
    'class_is_pulled_right',
    'class_is_clickable',
    'class_is_unselectable',
    'class_is_relative',
]
