"""
Module for LibreOffice backend (calc, writer, etc)
"""

from file_conversor.backend.office.calc_backend import *
from file_conversor.backend.office.impress_backend import *
from file_conversor.backend.office.writer_backend import *
