# src\file_conversor\cli\ppt\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "ppt"

# SUBCOMMANDS
CONVERT_NAME = "convert"

__all__ = [
    "COMMAND_NAME",

    "CONVERT_NAME",
]
