# src\file_conversor\cli\gui\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "gui"

# SUBCOMMANDS
START_NAME = "start"

__all__ = [
    "COMMAND_NAME",

    "START_NAME",
]
