# src/file_conversor/backend/gui/_components/margin.py

class MarginAll:
    M_AUTO = "m-auto"
    M_0 = "m-0"
    M_1 = "m-1"
    M_2 = "m-2"
    M_3 = "m-3"
    M_4 = "m-4"
    M_5 = "m-5"
    M_6 = "m-6"


class MarginX:
    M_AUTO = "mx-auto"
    M_0 = "mx-0"
    M_1 = "mx-1"
    M_2 = "mx-2"
    M_3 = "mx-3"
    M_4 = "mx-4"
    M_5 = "mx-5"
    M_6 = "mx-6"


class MarginY:
    M_AUTO = "my-auto"
    M_0 = "my-0"
    M_1 = "my-1"
    M_2 = "my-2"
    M_3 = "my-3"
    M_4 = "my-4"
    M_5 = "my-5"
    M_6 = "my-6"


class MarginLeft:
    M_AUTO = "ml-auto"
    M_0 = "ml-0"
    M_1 = "ml-1"
    M_2 = "ml-2"
    M_3 = "ml-3"
    M_4 = "ml-4"
    M_5 = "ml-5"
    M_6 = "ml-6"


class MarginRight:
    M_AUTO = "mr-auto"
    M_0 = "mr-0"
    M_1 = "mr-1"
    M_2 = "mr-2"
    M_3 = "mr-3"
    M_4 = "mr-4"
    M_5 = "mr-5"
    M_6 = "mr-6"


class MarginTop:
    M_AUTO = "mt-auto"
    M_0 = "mt-0"
    M_1 = "mt-1"
    M_2 = "mt-2"
    M_3 = "mt-3"
    M_4 = "mt-4"
    M_5 = "mt-5"
    M_6 = "mt-6"


class MarginBottom:
    M_AUTO = "mb-auto"
    M_0 = "mb-0"
    M_1 = "mb-1"
    M_2 = "mb-2"
    M_3 = "mb-3"
    M_4 = "mb-4"
    M_5 = "mb-5"
    M_6 = "mb-6"


__all__ = [
    'MarginAll',
    'MarginX',
    'MarginY',
    'MarginTop',
    'MarginBottom',
    'MarginLeft',
    'MarginRight',
]
