// src\file_conversor\.web\static\js\utils\__init__.js

import windowEventHandler from "./window_event_handler.js";
import documentEventHandler from "./document_event_handler.js";

export {
    windowEventHandler,
    documentEventHandler,
};